import numpy as np
import json
h_H0=0.6727   ## Oguri et al. 2018

Gpc3            = 1.e9
R0h67           = 22./Gpc3 # Mpc^3 / yr # Local value of merger rate
R0              = R0h67/h_H0**3 ### convert rate to h^3/Mpc^3/yr

############## 
## Source merger rate density models
############## 

RUnnormed = lambda z, kappa, gamma, zp: (1.+z)**kappa/(1.+((1.+z)/(1.+zp))**(gamma+kappa))
Rz        = lambda z, kappa, gamma, zp, R0: R0 * RUnnormed(z,kappa,gamma,zp) / RUnnormed(0,kappa,gamma,zp)
############## 
## Oguri's fit to the Belczynski merger rate density R(z) ; Oguri et al. 2018
a1, a2, a3, a4     = 6.6e3, 1.6, 2.1, 30
RzOguriUnnormed    = lambda z: a1*np.exp(a2*z)/(np.exp(a3*z)+a4) /Gpc3 # Oguri fit to the Belczynski merger rate density
RzOguri            = lambda z: R0 * RzOguriUnnormed(z) / RzOguriUnnormed(0) # Normalize it so that it agrees with the local merger rate

############## 
## Power Law Models
RzFlat             = lambda z: R0          # Flat merger-rate density
RzOne              = lambda z: R0*(1+z)    # \propto (1+z) merger-rate density
RzTwo              = lambda z: R0*(1+z)**2 # \propto (1+z)^2
RzThree            = lambda z: R0*(1+z)**3 # \propto (1+z)^3

RzAlpha            = lambda z, R0, alpha: R0*(1+z)**alpha # Free-form. We will use this for the R&P results.
############## 
## Ken's fit to the Belczynski distribution
def merger_rate_density_unnorm(z,alpha=2.7,beta=5.6,C=2.9):
    zp1=1+z
    return zp1**(alpha)/(1+(zp1/C)**beta)

alpha,beta,C=2.57,5.83,3.36
RzKen = lambda z: R0*merger_rate_density_unnorm(z,alpha,beta,C) / merger_rate_density_unnorm(0,alpha,beta,C)
############## 

RzPopSynthMin = lambda z: (58.49777737168871*np.exp(2.0642418004248095*z))/(2.823384141940439 + 240867.52946088542**(0.2040816326530612*z))
RzPopSynthMax = lambda z: (105355.78446144641*np.exp(1.3027814411130725*z))/(2714.3552696249076 + 4.792543342092949e9**(z/10.))
RzSynth_norm_min=lambda z:  RzPopSynthMin(z)/h_H0**3/Gpc3
RzSynth_norm_max=lambda z:  RzPopSynthMax(z)/h_H0**3/Gpc3

###########

